@@ -43,7 +43,7 @@ def test_single_user_transition():
     assert df.user_transition('oh hey there', df.state()) == States.C
 
 def test_system_transition():
-    for _ in range(20):
+    for _ in range(6):
         df = DialogueFlow(States.A)
         df.add_system_transition(States.A, States.B, 'hello', score=2.0)
         df.add_system_transition(States.A, States.C, 'hey')
@@ -60,6 +60,32 @@ def test_user_transition():
     assert df.user_transition('oh hey there', df.state()) == States.C
     assert df.user_transition('well see ya later', df.state()) == States.D
 
+def test_user_transition_list_disjunction():
+    df = DialogueFlow('root', initial_speaker=Speaker.USER)
+    nlu = [
+        'hello',
+        'hi',
+        'how are you'
+    ]
+    df.add_user_transition('root', 'x', nlu)
+    df.update_state_settings('root', error_successor='y')
+    df.user_turn('hi')
+    assert df.state() == 'x'
+
+def test_global_transition_list_disjunction():
+    df = DialogueFlow('root', initial_speaker=Speaker.USER)
+    nlu = [
+        'hi',
+        'hello'
+    ]
+    df.add_state('root', 'x')
+    df.add_system_transition('x', 'y', 'hello')
+    df.add_global_nlu('x', nlu)
+    df.user_turn('ok')
+    df.system_turn()
+    df.user_turn('hi')
+    assert df.state() == 'x'
+
 def test_check():
     df = DialogueFlow(States.A)
     df.add_state(States.B, error_successor=States.C)
@@ -85,26 +111,6 @@ def test_check():
     df.add_system_transition(States.C, States.D, 'ok then $X alright')
     assert not df.check()
 
-def test_global_transitions():
-    df = DialogueFlow(States.A)
-    df.add_state(States.A, global_nlu='{hi, hey, hello}')
-    df.add_state(States.B, error_successor=States.C)
-    df.add_state(States.D, error_successor=States.D)
-    df.add_system_transition(States.A, States.B, 'hello')
-    df.add_user_transition(States.B, States.C, '[how, you]')
-    df.add_user_transition(States.B, States.A, '[{bye, goodbye, see you, see ya, later}]')
-    df.add_system_transition(States.C, States.D, 'ok')
-    assert df.transition_natex(States.D, States.A, Speaker.USER).expression() == '{hi, hey, hello}'
-    df = DialogueFlow(States.A)
-    df.add_state(States.B, error_successor=States.C)
-    df.add_state(States.D, error_successor=States.D)
-    df.add_system_transition(States.A, States.B, 'hello')
-    df.add_user_transition(States.B, States.C, '[how, you]')
-    df.add_user_transition(States.B, States.A, '[{bye, goodbye, see you, see ya, later}]')
-    df.add_system_transition(States.C, States.D, 'ok')
-    df.update_state_settings(States.A, global_nlu='{hi, hey, hello}')
-    assert df.transition_natex(States.D, States.A, Speaker.USER).expression() == '{hi, hey, hello}'
-
 def test_user_multi_hop():
     df = DialogueFlow(States.A, initial_speaker=Speaker.USER)
     df.add_state(States.B, error_successor=States.B, user_multi_hop=True)
@@ -224,7 +230,7 @@ def test_reset():
     assert df.state() == States.E
 
     df.reset()
-    assert df.state() == States.A
+    assert df.state() == 'States.A'
     assert df.speaker() == Speaker.SYSTEM
     response = df.system_turn()
     df.user_turn("")
@@ -263,4 +269,193 @@ def test_information_state_based_dialogue():
     assert df.system_turn(debugging=True).strip() == 'i am a stupid state machine'
 
 
+def test_transitions_to_transitions():
+    df = DialogueFlow('root', initial_speaker=Speaker.SYSTEM)
+    transitions = {
+        'state': 'root',
+
+        '"Hello World!"': {
+            'state': 'state_2',
+
+            'error': {
+                '"Hi,"': 'root->state_2'
+            }
+        }
+    }
+    df.load_transitions(transitions, speaker=Speaker.SYSTEM)
+    assert df.system_turn() == 'Hello World!'
+    df.user_turn('blah')
+    assert df.system_turn() == 'Hi, Hello World!'
+
+
+def test_dialogue_flow_modular_transition():
+    # test to ensure dialogue flow module with transition to other
+    # component does not crash when tested indpendently
+    df = DialogueFlow('root', initial_speaker=Speaker.SYSTEM)
+    transitions = {
+        'state': 'root',
+
+        '"Hello World!"': {
+            'state': 'state_2',
+
+            'error': {
+                '"How are you?"': {
+                    'state': 'mystate',
+                    'error': 'root'
+                },
+                '"External thing"':{
+                    'state': 'external_component:external_state',
+                    'score': 2.0
+                }
+            }
+        }
+    }
+    df.load_transitions(transitions, speaker=Speaker.SYSTEM)
+    assert df.system_turn() == 'Hello World!'
+    df.user_turn('blah')
+    assert df.system_turn() == 'How are you?'
+    assert df.state() == 'mystate'
+
+
+def test_information_state_state_set():
+    df = DialogueFlow('root', initial_speaker=Speaker.USER)
+    transitions = {
+        'state': 'root',
+        'error': {
+            'state': 'one',
+            'okay': {
+                'state': 'two',
+                'error': {
+                    'state': 'three',
+                    'sure': 'root'
+                }
+            }
+        },
+        'something': {
+            'state': 'special',
+            'that is great': 'root'
+        }
+    }
+    df.add_update_rule('[{read, watched}]', '#TRANSITION(special, 2.0)')
+    df.load_transitions(transitions, speaker=Speaker.USER)
+    df.user_turn('hi')
+    assert df.system_turn() == 'okay'
+    df.user_turn('i read a book')
+    assert df.state() == 'special'
+    assert df.system_turn() == 'that is great'
+    assert df.state() == 'root'
+
+def test_unexpected_input_macro():
+    df = DialogueFlow('root', initial_speaker=Speaker.USER)
+    df.add_user_transition('root', 'x', '#UNX')
+    df.add_system_transition('x', 'y', '"So, whats for dinner?"')
+    df.user_turn('blah')
+    tokens = df.system_turn().split()
+    assert len(tokens) > 4 and 'So, whats for dinner?' == ' '.join(tokens[-4:])
+
+def test_transition_stack():
+    df = DialogueFlow('root')
+    transitions = {
+        'state': 'root',
+        'hello':{
+            'error':{
+                'how are you':{
+                    'error':{
+                        'state': 'special',
+                        'switch': True,
+                        'so':{
+                            'state': 'x',
+                            'score': 3.0
+                        },
+                        'well': {
+                            'state': 'y',
+                            'score': 2.0
+                        },
+                        'hmm': 'a->b'
+                    }
+                }
+            },
+            'other':{
+                'state': 'x',
+                'family is good':{
+                    'error':{
+                        'switch': True
+                    }
+                }
+            },
+            'next':{
+                'state': 'y',
+                'what do you think':{
+                    'error':{
+                        'switch': True
+                    }
+                }
+            },
+            'strange':{
+                'state': 'a',
+                'pets are nice': {
+                    'state': 'b',
+                    'error':
+                    {
+                        'switch': True
+                    }
+                }
+            }
+        }
+    }
+    df.load_transitions(transitions)
+    assert df.system_turn() == 'hello'
+    df.user_turn('hi')
+    assert df.system_turn() == 'how are you'
+    df.user_turn('fine')
+    response = df.system_turn()
+    if response == 'so family is good':
+        df.user_turn('yeah')
+        response = df.system_turn()
+        if response == 'pets are nice':
+            df.user_turn('blah')
+            assert df.system_turn() == 'what do you think'
+        elif response == 'what do you think':
+            df.user_turn('blah')
+            assert df.system_turn() == 'pets are nice'
+
+
+def test_global_transition_priority():
+    df = DialogueFlow('root')
+    transitions = {
+        'state': 'root',
+        'hello':{
+            '[oh]':{
+                'score': 0.6,
+                'okay':{}
+            },
+            '[wow]':{
+                'score': 2.0,
+                'alright': {}
+            }
+        }
+    }
+    df.add_global_nlu('root', '/.*/', score=0.7)
+    df.load_transitions(transitions)
+    df.system_turn()
+    df.user_turn('oh wow')
+    assert df.system_turn() == 'alright'
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
